@@ -17,38 +17,38 @@ zone=example.
 zonefile=example.db
 signedfile=example.db.signed
 
-ksk=`$KEYGEN -q -a RSASHA256 -b 2048 -fk $zone`
-zsk=`$KEYGEN -q -a RSASHA256 -b 1024 $zone`
+ksk=$($KEYGEN -q -a ${DEFAULT_ALGORITHM} -b ${DEFAULT_BITS} -fk $zone)
+zsk=$($KEYGEN -q -a ${DEFAULT_ALGORITHM} -b ${DEFAULT_BITS} $zone)
 $SIGNER -S -o $zone -f $signedfile $zonefile > /dev/null
 
 zone=wildcard-secure.example.
 zonefile=wildcard-secure.db
 signedfile=wildcard-secure.example.db.signed
 
-ksk=`$KEYGEN -q -a RSASHA256 -b 2048 -fk $zone`
-zsk=`$KEYGEN -q -a RSASHA256 -b 1024 $zone`
+ksk=$($KEYGEN -q -a ${DEFAULT_ALGORITHM} -b ${DEFAULT_BITS} -fk $zone)
+zsk=$($KEYGEN -q -a ${DEFAULT_ALGORITHM} -b ${DEFAULT_BITS} $zone)
 $SIGNER -S -o $zone -f $signedfile $zonefile > /dev/null
 
 zone=wildcard-nsec.example.
 zonefile=wildcard.db
 signedfile=wildcard-nsec.example.db.signed
 
-ksk=`$KEYGEN -q -a RSASHA256 -b 2048 -fk $zone`
-zsk=`$KEYGEN -q -a RSASHA256 -b 1024 $zone`
+ksk=$($KEYGEN -q -a ${DEFAULT_ALGORITHM} -b ${DEFAULT_BITS} -fk $zone)
+zsk=$($KEYGEN -q -a ${DEFAULT_ALGORITHM} -b ${DEFAULT_BITS} $zone)
 $SIGNER -S -o $zone -f $signedfile $zonefile > /dev/null
 
 zone=wildcard-nsec3.example.
 zonefile=wildcard.db
 signedfile=wildcard-nsec3.example.db.signed
 
-ksk=`$KEYGEN -q -a RSASHA256 -b 2048 -fk $zone`
-zsk=`$KEYGEN -q -a RSASHA256 -b 1024 $zone`
+ksk=$($KEYGEN -q -a ${DEFAULT_ALGORITHM} -b ${DEFAULT_BITS} -fk $zone)
+zsk=$($KEYGEN -q -a ${DEFAULT_ALGORITHM} -b ${DEFAULT_BITS} $zone)
 $SIGNER -S -3 - -H 0 -o $zone -f $signedfile $zonefile > /dev/null
 
 zone=wildcard-nsec3-optout.example.
 zonefile=wildcard.db
 signedfile=wildcard-nsec3-optout.example.db.signed
 
-ksk=`$KEYGEN -q -a RSASHA256 -b 2048 -fk $zone`
-zsk=`$KEYGEN -q -a RSASHA256 -b 1024 $zone`
+ksk=$($KEYGEN -q -a ${DEFAULT_ALGORITHM} -b ${DEFAULT_BITS} -fk $zone)
+zsk=$($KEYGEN -q -a ${DEFAULT_ALGORITHM} -b ${DEFAULT_BITS} $zone)
 $SIGNER -S -3 - -H 0 -A -o $zone -f $signedfile $zonefile > /dev/null
