@@ -14,6 +14,7 @@
 #include <stdbool.h>
 #include <stddef.h>
 
+#include <isc/async.h>
 #include <isc/buffer.h>
 #include <isc/md.h>
 #include <isc/mem.h>
@@ -24,14 +25,12 @@
 #include <isc/result.h>
 #include <isc/safe.h>
 #include <isc/sockaddr.h>
-#include <isc/task.h>
 #include <isc/util.h>
 
 #include <dns/adb.h>
 #include <dns/client.h>
 #include <dns/db.h>
 #include <dns/dispatch.h>
-#include <dns/events.h>
 #include <dns/forward.h>
 #include <dns/keytable.h>
 #include <dns/message.h>
@@ -44,7 +43,6 @@
 #include <dns/rdatatype.h>
 #include <dns/request.h>
 #include <dns/resolver.h>
-#include <dns/tsec.h>
 #include <dns/tsig.h>
 #include <dns/view.h>
 
@@ -76,10 +74,8 @@ struct dns_client {
 	unsigned int magic;
 	unsigned int attributes;
 	isc_mem_t *mctx;
-	isc_taskmgr_t *taskmgr;
-	isc_task_t *task;
+	isc_loop_t *loop;
 	isc_nm_t *nm;
-	isc_loopmgr_t *loopmgr;
 	dns_dispatchmgr_t *dispatchmgr;
 	dns_dispatch_t *dispatchv4;
 	dns_dispatch_t *dispatchv6;
@@ -108,7 +104,6 @@ typedef struct resctx {
 	bool want_tcp;
 
 	ISC_LINK(struct resctx) link;
-	isc_task_t *task;
 	dns_view_t *view;
 	unsigned int restarts;
 	dns_fixedname_t name;
@@ -116,7 +111,7 @@ typedef struct resctx {
 	dns_fetch_t *fetch;
 	dns_namelist_t namelist;
 	isc_result_t result;
-	dns_clientresevent_t *event;
+	dns_clientresume_t *rev;
 	dns_rdataset_t *rdataset;
 	dns_rdataset_t *sigrdataset;
 } resctx_t;
@@ -137,7 +132,7 @@ typedef struct resarg {
 } resarg_t;
 
 static void
-client_resfind(resctx_t *rctx, dns_fetchevent_t *event);
+client_resfind(resctx_t *rctx, dns_fetchresponse_t *event);
 static void
 destroyrestrans(dns_clientrestrans_t **transp);
 
@@ -205,9 +200,9 @@ getudpdispatch(int family, dns_dispatchmgr_t *dispatchmgr,
 
 static isc_result_t
 createview(isc_mem_t *mctx, dns_rdataclass_t rdclass, isc_loopmgr_t *loopmgr,
-	   isc_taskmgr_t *taskmgr, isc_nm_t *nm, dns_dispatchmgr_t *dispatchmgr,
-	   dns_dispatch_t *dispatchv4, dns_dispatch_t *dispatchv6,
-	   dns_view_t **viewp) {
+	   isc_nm_t *nm, isc_tlsctx_cache_t *tlsctx_client_cache,
+	   dns_dispatchmgr_t *dispatchmgr, dns_dispatch_t *dispatchv4,
+	   dns_dispatch_t *dispatchv6, dns_view_t **viewp) {
 	isc_result_t result;
 	dns_view_t *view = NULL;
 
@@ -216,14 +211,17 @@ createview(isc_mem_t *mctx, dns_rdataclass_t rdclass, isc_loopmgr_t *loopmgr,
 		return (result);
 	}
 
+	dns_view_setdispatchmgr(view, dispatchmgr);
+
 	/* Initialize view security roots */
 	result = dns_view_initsecroots(view, mctx);
 	if (result != ISC_R_SUCCESS) {
 		goto cleanup_view;
 	}
 
-	result = dns_view_createresolver(view, loopmgr, taskmgr, 1, nm, 0,
-					 dispatchmgr, dispatchv4, dispatchv6);
+	result = dns_view_createresolver(view, loopmgr, 1, nm, 0,
+					 tlsctx_client_cache, dispatchv4,
+					 dispatchv6);
 	if (result != ISC_R_SUCCESS) {
 		goto cleanup_view;
 	}
@@ -243,8 +241,8 @@ cleanup_view:
 }
 
 isc_result_t
-dns_client_create(isc_mem_t *mctx, isc_loopmgr_t *loopmgr,
-		  isc_taskmgr_t *taskmgr, isc_nm_t *nm, unsigned int options,
+dns_client_create(isc_mem_t *mctx, isc_loopmgr_t *loopmgr, isc_nm_t *nm,
+		  unsigned int options, isc_tlsctx_cache_t *tlsctx_client_cache,
 		  dns_client_t **clientp, const isc_sockaddr_t *localaddr4,
 		  const isc_sockaddr_t *localaddr6) {
 	isc_result_t result;
@@ -254,27 +252,21 @@ dns_client_create(isc_mem_t *mctx, isc_loopmgr_t *loopmgr,
 	dns_view_t *view = NULL;
 
 	REQUIRE(mctx != NULL);
-	REQUIRE(taskmgr != NULL);
 	REQUIRE(nm != NULL);
+	REQUIRE(tlsctx_client_cache != NULL);
 	REQUIRE(clientp != NULL && *clientp == NULL);
 
 	UNUSED(options);
 
 	client = isc_mem_get(mctx, sizeof(*client));
 	*client = (dns_client_t){
-		.taskmgr = taskmgr,
-		.loopmgr = loopmgr,
+		.loop = isc_loop_get(loopmgr, 0),
 		.nm = nm,
 	};
 
-	result = isc_task_create(client->taskmgr, &client->task, 0);
-	if (result != ISC_R_SUCCESS) {
-		goto cleanup_lock;
-	}
-
 	result = dns_dispatchmgr_create(mctx, nm, &client->dispatchmgr);
 	if (result != ISC_R_SUCCESS) {
-		goto cleanup_task;
+		goto cleanup_client;
 	}
 	(void)setsourceports(mctx, client->dispatchmgr);
 
@@ -309,8 +301,9 @@ dns_client_create(isc_mem_t *mctx, isc_loopmgr_t *loopmgr,
 	isc_refcount_init(&client->references, 1);
 
 	/* Create the default view for class IN */
-	result = createview(mctx, dns_rdataclass_in, loopmgr, taskmgr, nm,
-			    client->dispatchmgr, dispatchv4, dispatchv6, &view);
+	result = createview(mctx, dns_rdataclass_in, loopmgr, nm,
+			    tlsctx_client_cache, client->dispatchmgr,
+			    dispatchv4, dispatchv6, &view);
 	if (result != ISC_R_SUCCESS) {
 		goto cleanup_references;
 	}
@@ -343,9 +336,7 @@ cleanup_dispatchmgr:
 		dns_dispatch_detach(&dispatchv6);
 	}
 	dns_dispatchmgr_detach(&client->dispatchmgr);
-cleanup_task:
-	isc_task_detach(&client->task);
-cleanup_lock:
+cleanup_client:
 	isc_mem_put(mctx, client, sizeof(*client));
 
 	return (result);
@@ -366,8 +357,6 @@ destroyclient(dns_client_t *client) {
 
 	dns_dispatchmgr_detach(&client->dispatchmgr);
 
-	isc_task_detach(&client->task);
-
 	client->magic = 0;
 
 	isc_mem_putanddetach(&client->mctx, client, sizeof(*client));
@@ -457,16 +446,13 @@ putrdataset(isc_mem_t *mctx, dns_rdataset_t **rdatasetp) {
 }
 
 static void
-fetch_done(isc_task_t *task, isc_event_t *event) {
-	resctx_t *rctx = event->ev_arg;
-	dns_fetchevent_t *fevent;
+fetch_done(void *arg) {
+	dns_fetchresponse_t *resp = (dns_fetchresponse_t *)arg;
+	resctx_t *rctx = resp->arg;
 
-	REQUIRE(event->ev_type == DNS_EVENT_FETCHDONE);
 	REQUIRE(RCTX_VALID(rctx));
-	REQUIRE(rctx->task == task);
-	fevent = (dns_fetchevent_t *)event;
 
-	client_resfind(rctx, fevent);
+	client_resfind(rctx, resp);
 }
 
 static isc_result_t
@@ -493,8 +479,8 @@ start_fetch(resctx_t *rctx) {
 	result = dns_resolver_createfetch(
 		rctx->view->resolver, dns_fixedname_name(&rctx->name),
 		rctx->type, NULL, NULL, NULL, NULL, 0, fopts, 0, NULL,
-		rctx->task, fetch_done, rctx, rctx->rdataset, rctx->sigrdataset,
-		&rctx->fetch);
+		rctx->client->loop, fetch_done, rctx, rctx->rdataset,
+		rctx->sigrdataset, &rctx->fetch);
 
 	return (result);
 }
@@ -520,8 +506,8 @@ view_find(resctx_t *rctx, dns_db_t **dbp, dns_dbnode_t **nodep,
 }
 
 static void
-client_resfind(resctx_t *rctx, dns_fetchevent_t *event) {
-	isc_mem_t *mctx;
+client_resfind(resctx_t *rctx, dns_fetchresponse_t *resp) {
+	isc_mem_t *mctx = NULL;
 	isc_result_t tresult, result = ISC_R_SUCCESS;
 	isc_result_t vresult = ISC_R_SUCCESS;
 	bool want_restart;
@@ -551,7 +537,7 @@ client_resfind(resctx_t *rctx, dns_fetchevent_t *event) {
 		rctx->restarts++;
 		want_restart = false;
 
-		if (event == NULL) {
+		if (resp == NULL) {
 			fname = dns_fixedname_initname(&foundname);
 			INSIST(!dns_rdataset_isassociated(rctx->rdataset));
 			INSIST(rctx->sigrdataset == NULL ||
@@ -581,16 +567,17 @@ client_resfind(resctx_t *rctx, dns_fetchevent_t *event) {
 				goto done;
 			}
 		} else {
-			INSIST(event != NULL);
-			INSIST(event->fetch == rctx->fetch);
+			INSIST(resp != NULL);
+			INSIST(resp->fetch == rctx->fetch);
 			dns_resolver_destroyfetch(&rctx->fetch);
-			db = event->db;
-			node = event->node;
-			result = event->result;
-			vresult = event->vresult;
-			fname = event->foundname;
-			INSIST(event->rdataset == rctx->rdataset);
-			INSIST(event->sigrdataset == rctx->sigrdataset);
+			db = resp->db;
+			node = resp->node;
+			result = resp->result;
+			vresult = resp->vresult;
+			fname = resp->foundname;
+			INSIST(resp->rdataset == rctx->rdataset);
+			INSIST(resp->sigrdataset == rctx->sigrdataset);
+			isc_mem_putanddetach(&resp->mctx, resp, sizeof(*resp));
 		}
 
 		/*
@@ -718,7 +705,7 @@ client_resfind(resctx_t *rctx, dns_fetchevent_t *event) {
 			int n = 0;
 			dns_rdatasetiter_t *rdsiter = NULL;
 
-			tresult = dns_db_allrdatasets(db, node, NULL, 0,
+			tresult = dns_db_allrdatasets(db, node, NULL, 0, 0,
 						      &rdsiter);
 			if (tresult != ISC_R_SUCCESS) {
 				result = tresult;
@@ -745,7 +732,8 @@ client_resfind(resctx_t *rctx, dns_fetchevent_t *event) {
 				tresult = dns_rdatasetiter_next(rdsiter);
 
 				if (tresult == ISC_R_SUCCESS &&
-				    rctx->rdataset == NULL) {
+				    rctx->rdataset == NULL)
+				{
 					tresult = getrdataset(mctx,
 							      &rctx->rdataset);
 					if (tresult != ISC_R_SUCCESS) {
@@ -804,7 +792,8 @@ client_resfind(resctx_t *rctx, dns_fetchevent_t *event) {
 			dns_rdataset_t *rdataset;
 
 			while ((rdataset = ISC_LIST_HEAD(ansname->list)) !=
-			       NULL) {
+			       NULL)
+			{
 				ISC_LIST_UNLINK(ansname->list, rdataset, link);
 				putrdataset(mctx, &rdataset);
 			}
@@ -818,9 +807,6 @@ client_resfind(resctx_t *rctx, dns_fetchevent_t *event) {
 		if (db != NULL) {
 			dns_db_detach(&db);
 		}
-		if (event != NULL) {
-			isc_event_free(ISC_EVENT_PTR(&event));
-		}
 
 		/*
 		 * Limit the number of restarts.
@@ -854,30 +840,24 @@ client_resfind(resctx_t *rctx, dns_fetchevent_t *event) {
 	} while (want_restart);
 
 	if (send_event) {
-		isc_task_t *task;
-
 		while ((name = ISC_LIST_HEAD(rctx->namelist)) != NULL) {
 			ISC_LIST_UNLINK(rctx->namelist, name, link);
-			ISC_LIST_APPEND(rctx->event->answerlist, name, link);
+			ISC_LIST_APPEND(rctx->rev->answerlist, name, link);
 		}
 
-		rctx->event->result = result;
-		rctx->event->vresult = vresult;
-		task = rctx->event->ev_sender;
-		rctx->event->ev_sender = rctx;
-		isc_task_sendanddetach(&task, ISC_EVENT_PTR(&rctx->event));
+		rctx->rev->result = result;
+		rctx->rev->vresult = vresult;
+		isc_async_run(rctx->client->loop, rctx->rev->cb, rctx->rev);
 	}
 }
 
 static void
-resolve_done(isc_task_t *task, isc_event_t *event) {
-	resarg_t *resarg = event->ev_arg;
-	dns_clientresevent_t *rev = (dns_clientresevent_t *)event;
+resolve_done(void *arg) {
+	dns_clientresume_t *rev = (dns_clientresume_t *)arg;
+	resarg_t *resarg = rev->arg;
 	dns_name_t *name = NULL;
 	isc_result_t result;
 
-	UNUSED(task);
-
 	resarg->result = rev->result;
 	resarg->vresult = rev->vresult;
 	while ((name = ISC_LIST_HEAD(rev->answerlist)) != NULL) {
@@ -885,8 +865,7 @@ resolve_done(isc_task_t *task, isc_event_t *event) {
 		ISC_LIST_APPEND(*resarg->namelist, name, link);
 	}
 
-	isc_event_free(&event);
-
+	isc_mem_put(resarg->mctx, rev, sizeof(*rev));
 	destroyrestrans(&resarg->trans);
 
 	result = resarg->result;
@@ -933,8 +912,7 @@ dns_client_resolve(dns_client_t *client, const dns_name_t *name,
 	isc_mem_attach(client->mctx, &resarg->mctx);
 
 	result = dns_client_startresolve(client, name, rdclass, type, options,
-					 client->task, resolve_done, resarg,
-					 &resarg->trans);
+					 resolve_done, resarg, &resarg->trans);
 	if (result != ISC_R_SUCCESS) {
 		isc_mem_put(client->mctx, resarg, sizeof(*resarg));
 		return (result);
@@ -946,15 +924,13 @@ dns_client_resolve(dns_client_t *client, const dns_name_t *name,
 isc_result_t
 dns_client_startresolve(dns_client_t *client, const dns_name_t *name,
 			dns_rdataclass_t rdclass, dns_rdatatype_t type,
-			unsigned int options, isc_task_t *task,
-			isc_taskaction_t action, void *arg,
+			unsigned int options, isc_job_cb cb, void *arg,
 			dns_clientrestrans_t **transp) {
-	dns_clientresevent_t *event = NULL;
+	dns_clientresume_t *rev = NULL;
 	resctx_t *rctx = NULL;
-	isc_task_t *tclone = NULL;
-	isc_mem_t *mctx;
+	isc_mem_t *mctx = NULL;
 	isc_result_t result;
-	dns_rdataset_t *rdataset, *sigrdataset;
+	dns_rdataset_t *rdataset = NULL, *sigrdataset = NULL;
 	bool want_dnssec, want_validation, want_cdflag, want_tcp;
 
 	REQUIRE(DNS_CLIENT_VALID(client));
@@ -962,8 +938,6 @@ dns_client_startresolve(dns_client_t *client, const dns_name_t *name,
 	REQUIRE(rdclass == dns_rdataclass_in);
 
 	mctx = client->mctx;
-	rdataset = NULL;
-	sigrdataset = NULL;
 	want_dnssec = ((options & DNS_CLIENTRESOPT_NODNSSEC) == 0);
 	want_validation = ((options & DNS_CLIENTRESOPT_NOVALIDATE) == 0);
 	want_cdflag = ((options & DNS_CLIENTRESOPT_NOCDFLAG) == 0);
@@ -972,24 +946,25 @@ dns_client_startresolve(dns_client_t *client, const dns_name_t *name,
 	/*
 	 * Prepare some intermediate resources
 	 */
-	tclone = NULL;
-	isc_task_attach(task, &tclone);
-	event = (dns_clientresevent_t *)isc_event_allocate(
-		mctx, tclone, DNS_EVENT_CLIENTRESDONE, action, arg,
-		sizeof(*event));
-	event->result = DNS_R_SERVFAIL;
-	ISC_LIST_INIT(event->answerlist);
+	rev = isc_mem_get(mctx, sizeof(*rev));
+	*rev = (dns_clientresume_t){
+		.result = DNS_R_SERVFAIL,
+		.answerlist = ISC_LIST_INITIALIZER,
+		.cb = cb,
+		.arg = arg,
+	};
 
 	rctx = isc_mem_get(mctx, sizeof(*rctx));
 	*rctx = (resctx_t){
 		.client = client,
-		.task = client->task,
-		.event = event,
+		.rev = rev,
 		.type = type,
 		.want_dnssec = want_dnssec,
 		.want_validation = want_validation,
 		.want_cdflag = want_cdflag,
 		.want_tcp = want_tcp,
+		.namelist = ISC_LIST_INITIALIZER,
+		.link = ISC_LINK_INITIALIZER,
 	};
 
 	result = getrdataset(mctx, &rdataset);
@@ -1009,9 +984,7 @@ dns_client_startresolve(dns_client_t *client, const dns_name_t *name,
 	dns_fixedname_init(&rctx->name);
 	dns_name_copy(name, dns_fixedname_name(&rctx->name));
 
-	ISC_LINK_INIT(rctx, link);
 	dns_view_attach(client->view, &rctx->view);
-	ISC_LIST_INIT(rctx->namelist);
 
 	rctx->magic = RCTX_MAGIC;
 	isc_refcount_increment(&client->references);
@@ -1031,8 +1004,7 @@ cleanup:
 		putrdataset(client->mctx, &sigrdataset);
 	}
 	isc_mem_put(mctx, rctx, sizeof(*rctx));
-	isc_event_free(ISC_EVENT_PTR(&event));
-	isc_task_detach(&tclone);
+	isc_mem_put(mctx, rev, sizeof(*rev));
 
 	return (result);
 }
@@ -1075,7 +1047,6 @@ destroyrestrans(dns_clientrestrans_t **transp) {
 
 	REQUIRE(RCTX_VALID(rctx));
 	REQUIRE(rctx->fetch == NULL);
-	REQUIRE(rctx->event == NULL);
 
 	client = rctx->client;
 
@@ -1128,7 +1099,7 @@ dns_client_addtrustedkey(dns_client_t *client, dns_rdataclass_t rdclass,
 	dns_rdata_init(&rdata);
 	isc_buffer_setactive(databuf, isc_buffer_usedlength(databuf));
 	CHECK(dns_rdata_fromwire(&rdata, rdclass, rdtype, databuf,
-				 DNS_DECOMPRESS_NEVER, 0, &b));
+				 DNS_DECOMPRESS_NEVER, &b));
 
 	if (rdtype == dns_rdatatype_ds) {
 		CHECK(dns_rdata_tostruct(&rdata, &ds, NULL));
