@@ -28,6 +28,8 @@
 
 #include <isc/types.h>
 
+#include <dns/trace.h>
+
 typedef struct dns_acl	       dns_acl_t;
 typedef struct dns_aclelement  dns_aclelement_t;
 typedef struct dns_aclenv      dns_aclenv_t;
@@ -53,6 +55,8 @@ typedef void			       dns_clientupdatetrans_t;
 typedef struct dns_cache	       dns_cache_t;
 typedef uint16_t		       dns_cert_t;
 typedef struct dns_compress	       dns_compress_t;
+typedef enum dns_compress_flags	       dns_compress_flags_t;
+typedef struct dns_compress_slot       dns_compress_slot_t;
 typedef struct dns_db		       dns_db_t;
 typedef struct dns_dbimplementation    dns_dbimplementation_t;
 typedef struct dns_dbiterator	       dns_dbiterator_t;
@@ -65,7 +69,7 @@ typedef struct dns_dlzdb	       dns_dlzdb_t;
 typedef ISC_LIST(dns_dlzdb_t) dns_dlzdblist_t;
 typedef struct dns_dyndbctx	      dns_dyndbctx_t;
 typedef struct dns_sdlzimplementation dns_sdlzimplementation_t;
-typedef struct dns_decompress	      dns_decompress_t;
+typedef enum dns_decompress	      dns_decompress_t;
 typedef struct dns_dispatch	      dns_dispatch_t;
 typedef struct dns_dispatchlist	      dns_dispatchlist_t;
 typedef struct dns_dispatchset	      dns_dispatchset_t;
@@ -76,7 +80,6 @@ typedef ISC_LIST(dns_dns64_t) dns_dns64list_t;
 typedef struct dns_dnsseckey dns_dnsseckey_t;
 typedef ISC_LIST(dns_dnsseckey_t) dns_dnsseckeylist_t;
 typedef uint8_t			   dns_dsdigest_t;
-typedef uint8_t			   dns_dsyncscheme_t;
 typedef struct dns_dtdata	   dns_dtdata_t;
 typedef struct dns_dtenv	   dns_dtenv_t;
 typedef struct dns_dtmsg	   dns_dtmsg_t;
@@ -94,6 +97,8 @@ typedef struct dns_iptable	   dns_iptable_t;
 typedef uint32_t		   dns_iterations_t;
 typedef struct dns_kasp		   dns_kasp_t;
 typedef ISC_LIST(dns_kasp_t) dns_kasplist_t;
+typedef struct dns_kasp_digest dns_kasp_digest_t;
+typedef ISC_LIST(dns_kasp_digest_t) dns_kasp_digestlist_t;
 typedef struct dns_kasp_key dns_kasp_key_t;
 typedef ISC_LIST(dns_kasp_key_t) dns_kasp_keylist_t;
 typedef struct dns_kasp_nsec3param dns_kasp_nsec3param_t;
@@ -109,10 +114,8 @@ typedef uint64_t		   dns_masterstyle_flags_t;
 typedef struct dns_message	   dns_message_t;
 typedef uint16_t		   dns_messageid_t;
 typedef isc_region_t		   dns_label_t;
-typedef struct dns_lookup	   dns_lookup_t;
 typedef struct dns_name		   dns_name_t;
 typedef ISC_LIST(dns_name_t) dns_namelist_t;
-typedef struct dns_nta		  dns_nta_t;
 typedef struct dns_ntatable	  dns_ntatable_t;
 typedef uint16_t		  dns_opcode_t;
 typedef unsigned char		  dns_offsets_t[128];
@@ -127,29 +130,30 @@ typedef uint16_t		  dns_rdataclass_t;
 typedef struct dns_rdatalist	  dns_rdatalist_t;
 typedef struct dns_rdataset	  dns_rdataset_t;
 typedef ISC_LIST(dns_rdataset_t) dns_rdatasetlist_t;
-typedef struct dns_rdatasetiter	     dns_rdatasetiter_t;
-typedef uint16_t		     dns_rdatatype_t;
-typedef struct dns_request	     dns_request_t;
-typedef struct dns_requestmgr	     dns_requestmgr_t;
-typedef struct dns_resolver	     dns_resolver_t;
-typedef struct dns_sdbimplementation dns_sdbimplementation_t;
-typedef uint8_t			     dns_secalg_t;
-typedef uint8_t			     dns_secproto_t;
-typedef struct dns_signature	     dns_signature_t;
-typedef struct dns_sortlist_arg	     dns_sortlist_arg_t;
-typedef struct dns_ssurule	     dns_ssurule_t;
-typedef struct dns_ssutable	     dns_ssutable_t;
-typedef struct dns_stats	     dns_stats_t;
-typedef uint32_t		     dns_rdatastatstype_t;
-typedef struct dns_tkeyctx	     dns_tkeyctx_t;
-typedef uint16_t		     dns_trust_t;
-typedef struct dns_tsec		     dns_tsec_t;
-typedef struct dns_tsig_keyring	     dns_tsig_keyring_t;
-typedef struct dns_tsigkey	     dns_tsigkey_t;
-typedef uint32_t		     dns_ttl_t;
-typedef struct dns_update_state	     dns_update_state_t;
-typedef struct dns_validator	     dns_validator_t;
-typedef struct dns_view		     dns_view_t;
+typedef struct dns_rdatasetiter	  dns_rdatasetiter_t;
+typedef uint16_t		  dns_rdatatype_t;
+typedef struct dns_remote	  dns_remote_t;
+typedef struct dns_request	  dns_request_t;
+typedef struct dns_requestmgr	  dns_requestmgr_t;
+typedef struct dns_resolver	  dns_resolver_t;
+typedef uint8_t			  dns_secalg_t;
+typedef uint8_t			  dns_secproto_t;
+typedef struct dns_signature	  dns_signature_t;
+typedef struct dns_sortlist_arg	  dns_sortlist_arg_t;
+typedef struct dns_ssurule	  dns_ssurule_t;
+typedef struct dns_ssutable	  dns_ssutable_t;
+typedef struct dns_stats	  dns_stats_t;
+typedef uint32_t		  dns_rdatastatstype_t;
+typedef struct dns_tkeyctx	  dns_tkeyctx_t;
+typedef struct dns_transport	  dns_transport_t;
+typedef struct dns_transport_list dns_transport_list_t;
+typedef uint16_t		  dns_trust_t;
+typedef struct dns_tsig_keyring	  dns_tsig_keyring_t;
+typedef struct dns_tsigkey	  dns_tsigkey_t;
+typedef uint32_t		  dns_ttl_t;
+typedef struct dns_update_state	  dns_update_state_t;
+typedef struct dns_validator	  dns_validator_t;
+typedef struct dns_view		  dns_view_t;
 typedef ISC_LIST(dns_view_t) dns_viewlist_t;
 typedef struct dns_zone dns_zone_t;
 typedef ISC_LIST(dns_zone_t) dns_zonelist_t;
@@ -399,12 +403,13 @@ typedef void (*dns_loaddonefunc_t)(void *, isc_result_t);
 
 typedef void (*dns_rawdatafunc_t)(dns_zone_t *, dns_masterrawheader_t *);
 
-typedef isc_result_t (*dns_addrdatasetfunc_t)(void *, const dns_name_t *,
-					      dns_rdataset_t *);
+typedef isc_result_t (*dns_addrdatasetfunc_t)(void *arg, const dns_name_t *name,
+					      dns_rdataset_t *rdataset
+						      DNS__DB_FLARG);
 
-typedef isc_result_t (*dns_additionaldatafunc_t)(void *, const dns_name_t *,
-						 dns_rdatatype_t,
-						 dns_rdataset_t *);
+typedef isc_result_t (*dns_additionaldatafunc_t)(
+	void *arg, const dns_name_t *name, dns_rdatatype_t type,
+	dns_rdataset_t *rdataset DNS__DB_FLARG);
 
 typedef isc_result_t (*dns_digestfunc_t)(void *, isc_region_t *);
 
