@@ -11,7 +11,6 @@
  * information regarding copyright ownership.
  */
 
-#include <inttypes.h>
 #include <sched.h> /* IWYU pragma: keep */
 #include <setjmp.h>
 #include <stdarg.h>
@@ -95,7 +94,7 @@ rdata_fromparams(uint8_t hash, uint8_t flags, uint16_t iter, uint8_t saltlen,
 	nsec3param.iterations = iter;
 	nsec3param.salt_length = saltlen;
 	nsec3param.salt = salt;
-	return nsec3param;
+	return (nsec3param);
 }
 
 /*%
